
#include "br_can.h"

#define CAN_RATE_MAX_SUPPORT  4

typedef struct
{
  uint32_t bitrate;
  can_rsaw_type rsaw;
  can_bts1_type bts1;
  can_bts2_type bts2;
  uint16_t baudrate_div;
}can_rate_type;

/*APB1 108MHz*/
static can_rate_type can_rate_struct[CAN_RATE_MAX_SUPPORT] =
{
  {1000000, CAN_RSAW_3TQ, CAN_BTS1_8TQ, CAN_BTS2_3TQ, 9},
  {500000, CAN_RSAW_1TQ, CAN_BTS1_6TQ, CAN_BTS2_1TQ, 27},
  {250000, CAN_RSAW_2TQ, CAN_BTS1_13TQ, CAN_BTS2_2TQ, 27},
  {125000, CAN_RSAW_2TQ, CAN_BTS1_13TQ, CAN_BTS2_2TQ, 54},
};

static void br_can_init(void *pdata);
static void br_can_deinit(void *pdata);
static uint32_t br_can_tx(uint8_t *data, uint32_t len);
static uint8_t br_can_status(void);
br_handler br_can_handler = 
{
  br_can_init,
  br_can_deinit,
  br_can_tx,
  br_fifo_write,
  br_fifo_read,
  br_can_status,
};

static uint8_t can_tx_flag = 0;

/**
  * @brief  can initialization
  * @param  pdata: config data pointer
  * @retval none
  */
static void br_can_init(void *pdata)
{
  can_base_type can_base_struct;
  can_baudrate_type can_baudrate_struct;
  can_filter_init_type can_filter_init_struct;
  gpio_init_type gpio_init_struct;
  can_rate_type *p_rate = &can_rate_struct[1];
  uint8_t idx = 0;
  
  linecoding_type *linecoding = (linecoding_type *)pdata;
  
  /* clear can tx flag */
  can_tx_flag = 0;
  
  /* find support rate */
  for(idx = 0; idx < CAN_RATE_MAX_SUPPORT; idx ++)
  {
    if(linecoding->bitrate == can_rate_struct[idx].bitrate)
    {
      p_rate = &can_rate_struct[idx];
    }
  }
  
  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&gpio_init_struct);

  /* configure the can tx, rx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_MUX;
  gpio_init_struct.gpio_pins = GPIO_PINS_11 | GPIO_PINS_12;
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE;
  gpio_init(GPIOA, &gpio_init_struct);

  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE12, GPIO_MUX_9);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE11, GPIO_MUX_9);
  
  /* enable the can clock */
  crm_periph_clock_enable(CRM_CAN1_PERIPH_CLOCK, TRUE);
  
  can_reset(CAN1);
  
  /* can base init */
  can_default_para_init(&can_base_struct);
  can_base_struct.mode_selection = CAN_MODE_COMMUNICATE;
  can_base_struct.ttc_enable = FALSE;
  can_base_struct.aebo_enable = TRUE;
  can_base_struct.aed_enable = TRUE;
  can_base_struct.prsf_enable = FALSE;
  can_base_struct.mdrsel_selection = CAN_DISCARDING_FIRST_RECEIVED;
  can_base_struct.mmssr_selection = CAN_SENDING_BY_REQUEST;
  can_base_init(CAN1, &can_base_struct);
  
  /* can baudrate, set boudrate = pclk/(baudrate_div *(1 + bts1_size + bts2_size)) */
  can_baudrate_struct.baudrate_div = p_rate->baudrate_div;
  can_baudrate_struct.rsaw_size = p_rate->rsaw;
  can_baudrate_struct.bts1_size = p_rate->bts1;
  can_baudrate_struct.bts2_size = p_rate->bts2;
  can_baudrate_set(CAN1, &can_baudrate_struct);
  
  /* can filter init */
  can_filter_init_struct.filter_activate_enable = TRUE;
  can_filter_init_struct.filter_mode = CAN_FILTER_MODE_ID_MASK;
  can_filter_init_struct.filter_fifo = CAN_FILTER_FIFO0;
  can_filter_init_struct.filter_number = 0;
  can_filter_init_struct.filter_bit = CAN_FILTER_32BIT;
  can_filter_init_struct.filter_id_high = 0;
  can_filter_init_struct.filter_id_low = 0;
  can_filter_init_struct.filter_mask_high = 0;
  can_filter_init_struct.filter_mask_low = 0;
  can_filter_init(CAN1, &can_filter_init_struct);
  
  /* can interrupt config */
  nvic_irq_enable(CAN1_SE_IRQn, 0x01, 0x00);
  nvic_irq_enable(CAN1_RX0_IRQn, 0x01, 0x00);
  can_interrupt_enable(CAN1, CAN_RF0MIEN_INT, TRUE);

  /* error interrupt enable */
  can_interrupt_enable(CAN1, CAN_ETRIEN_INT, TRUE);
  can_interrupt_enable(CAN1, CAN_EOIEN_INT, TRUE);
  
}


static void br_can_deinit(void *pdata)
{
  gpio_init_type gpio_init_struct;

  gpio_default_para_init(&gpio_init_struct);

  /* configure the can tx, rx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_INPUT;
  gpio_init_struct.gpio_pins = GPIO_PINS_11 | GPIO_PINS_12;
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE;
  gpio_init(GPIOA, &gpio_init_struct);
  
  /* disable the can clock */
  can_reset(CAN1);
  
  crm_periph_clock_enable(CRM_CAN1_PERIPH_CLOCK, FALSE);
  
  /* clear can tx flag */
  can_tx_flag = 0;
}

/**
  * @brief  return can send status
  * @param  none
  * @retval none
  */
static uint8_t br_can_status(void)
{
  return 1;
}

/**
  *  @brief  can1 interrupt function rx0
  *  @param  none
  *  @retval none
  */
void CAN1_RX0_IRQHandler(void)
{
  can_rx_message_type rx_message_struct;
  uint8_t idx = 0, index = 0;
  static uint8_t rx_buffer[64] = {0};
  
  /* receivce data */
  if(can_interrupt_flag_get(CAN1,CAN_RF0MN_FLAG) != RESET)
  {
    can_message_receive(CAN1, CAN_RX_FIFO0, &rx_message_struct);
    if(rx_message_struct.id_type == CAN_ID_STANDARD)
    {
      rx_buffer[0] = 0;
      rx_buffer[1] = 0;
      rx_buffer[2] = (rx_message_struct.standard_id >> 8) & 0xFF;
      rx_buffer[3] = rx_message_struct.standard_id & 0xFF;
    }
    else
    {
      rx_buffer[0] = (rx_message_struct.extended_id >> 24) & 0xFF;  
      rx_buffer[1] = (rx_message_struct.extended_id >> 16) & 0xFF;
      rx_buffer[2] = (rx_message_struct.extended_id >> 8) & 0xFF;
      rx_buffer[3] = rx_message_struct.extended_id & 0xFF;     
    }
    rx_buffer[4] = rx_message_struct.id_type;
    rx_buffer[5] = rx_message_struct.frame_type;
    rx_buffer[6] = rx_message_struct.dlc;
    
    index = 7;
    for(idx = 0; idx < rx_message_struct.dlc ; idx ++)
    {
      rx_buffer[index ++] = rx_message_struct.data[idx];
    }
    br_can_handler.fifo_write(br_get_fifo(PERIPHERAL_TO_USB), rx_buffer, index);
  }
}

/**
  *  @brief  can1 interrupt function se
  *  @param  none
  *  @retval none
  */
void CAN1_SE_IRQHandler(void)
{
  if(can_interrupt_flag_get(CAN1,CAN_ETR_FLAG) != RESET)
  {
    can_flag_clear(CAN1, CAN_ETR_FLAG);
  }
}

/**
  * @brief  can send interface
  * @param  data: send data poiniter
  * @param  len: send data length
  * @retval number of data send
  */
static uint32_t br_can_tx(uint8_t *data, uint32_t len)
{
  uint8_t transmit_mailbox;
  uint8_t idx = 0, index = 0;
  uint32_t can_id = 0, timeout = 0xFFFFF;
  can_tx_message_type tx_message_struct;
  can_transmit_status_type status;
  
  can_tx_flag = 1;
  can_id = data[0] << 24 | data[1] << 16 | data[2] << 8 | data[3];
  tx_message_struct.standard_id = can_id & 0x7FF;
  tx_message_struct.extended_id = can_id;
  tx_message_struct.id_type = (can_identifier_type)data[4];
  tx_message_struct.frame_type = (can_trans_frame_type)data[5];
  tx_message_struct.dlc = data[6];
  index = 7;
  if(tx_message_struct.dlc > 8)
  {
    tx_message_struct.dlc = 8;
  }
  for(idx = 0; idx < tx_message_struct.dlc ; idx ++)
  {
    tx_message_struct.data[idx] = data[index ++];
  }
  
  transmit_mailbox = can_message_transmit(CAN1, &tx_message_struct);
  while(timeout -- && can_tx_flag)
  {
    status = can_transmit_status_get(CAN1, (can_tx_mailbox_num_type)transmit_mailbox);
    if(status == CAN_TX_STATUS_SUCCESSFUL)
    {
      break;
    }
    if(status == CAN_TX_STATUS_FAILED)
    {
      tx_message_struct.dlc = 0;
      break;
    }
  }
  can_transmit_cancel(CAN1, (can_tx_mailbox_num_type)transmit_mailbox);
  return tx_message_struct.dlc;
}

