
#include "br_fifo.h"

/**
  * @brief  fifo initialization
  * @param  p_fifo: initializes the fifo structure pointer
  * @retval none
  */
void br_fifo_init(br_fifo_type *p_fifo)
{
  if(p_fifo == NULL)
    return;
  
  /* set buffer mode start address */
  p_fifo->addr = (uint32_t)&p_fifo->data[0][0];
  
  /* default is empty and not full */
  p_fifo->is_empty = 1;
  p_fifo->is_full = 0;
  
  /* max fifo num */
  p_fifo->max_fifo = BR_MAX_FIFO;
  
  /* total buffer size */
  p_fifo->max_size = BR_MAX_FIFO_SIZE * BR_MAX_FIFO;
  
  /* read and write default position is 0 */
  p_fifo->rd_pos = 0;
  p_fifo->wr_pos = 0;
}

/**
  * @brief  fifo mode read the fifo data
  * @param  p_fifo: the fifo structure pointer
  * @param  data: stored the data pointer
  * @param  length: read data length 
  * @retval the length of read data
  */
uint32_t br_fifo_read(br_fifo_type *p_fifo, uint8_t *data, uint32_t length)
{
  if(p_fifo == NULL || length == 0 || data == 0)
  {
    return 0;
  }
  
  if(p_fifo->is_empty)
  {
    return 0;
  }
  
  /* check the fifo data length */
  if(p_fifo->data_len[p_fifo->rd_pos] < length)
  {
    length = p_fifo->data_len[p_fifo->rd_pos];
  }
  
  /* copy fifo data to buffer */
  memcpy(data, (uint8_t *)p_fifo->data[p_fifo->rd_pos], length);
  
  /* disable irq */
  __disable_irq();
  
  /* read position ++*/
  p_fifo->rd_pos ++;
  
  /* check the read pos is max fifo number */
  if(p_fifo->rd_pos >= p_fifo->max_fifo)
  {
    p_fifo->rd_pos = 0;
  }
  
  /* check the fifo is empty? */
  if(p_fifo->rd_pos == p_fifo->wr_pos)
  {
    p_fifo->is_empty = 1;
  }
  p_fifo->is_full = 0;
  /* enable irq */
  __enable_irq();
  
  return length;
}

/**
  * @brief  fifo mode write data to fifo
  * @param  p_fifo: the fifo structure pointer
  * @param  data: write data pointer
  * @param  length: write data length 
  * @retval the length of write data
  */
uint32_t br_fifo_write(br_fifo_type *p_fifo, uint8_t *data, uint32_t length)
{
  if(p_fifo == NULL || length == 0 || data == 0)
  {
    return 0;
  }
  
  if(p_fifo->is_full)
  {
    return 0;
  }
  
  if(length > BR_MAX_FIFO_SIZE)
  {
    length = BR_MAX_FIFO_SIZE;
  }
  
  /* the data length */
  p_fifo->data_len[p_fifo->wr_pos] = length;
  
  /* copy data from buffer to fifo */
  memcpy(((uint8_t *)p_fifo->data[p_fifo->wr_pos]), data, length);
  
  /* disable irq */
  __disable_irq();
  
  /* write position ++*/
  p_fifo->wr_pos ++;
  
  /* check the write pos is max fifo number */
  if(p_fifo->wr_pos >= p_fifo->max_fifo)
  {
    p_fifo->wr_pos = 0;
  }
  
  /* check the fifo is full? */
  if(p_fifo->rd_pos == p_fifo->wr_pos)
  {
    p_fifo->is_full = 1;
  }
  /* set the fifo to not empty */
  p_fifo->is_empty = 0;
  
  /* enable irq */
  __enable_irq();
  
  return length;
}

/**
  * @brief  get the fifo empty status
  * @param  p_fifo: the fifo structure pointer
  * @retval the fifo empty status
  */
uint8_t br_fifo_is_empty(br_fifo_type *p_fifo)
{
  return p_fifo->is_empty;
}

/**
  * @brief  get the fifo full status
  * @param  p_fifo: the fifo structure pointer
  * @retval the fifo full status
  */
uint8_t br_fifo_is_full(br_fifo_type *p_fifo)
{
  return p_fifo->is_full;
}

/**
  * @brief  buffer mode push data to fifo
  * @param  p_fifo: the fifo structure pointer
  * @param  data: push data
  * @retval push status 1 is OK, 0 is failed
  */
uint8_t br_buffer_line_push(br_fifo_type *p_fifo, uint8_t data)
{
  uint8_t *pdata = (uint8_t *)p_fifo->addr;
  uint32_t rsize;
  if(p_fifo == NULL)
  {
    return 0;
  }
  
  /* check the fifo is full? */
  if(p_fifo->is_full == 1 && p_fifo->rd_pos == p_fifo->wr_pos)
  {
    return 0;
  }
  
  /* copy data to fifo */
  pdata[p_fifo->wr_pos] = data;
  
  /* write pos ++ */
  p_fifo->wr_pos ++;
  
  /* check the write pos is max size */
  if(p_fifo->wr_pos >= p_fifo->max_size)
  {
    p_fifo->wr_pos = 0;
  }
  
  /* disable irq */
  __disable_irq();
  
  /* calculates the size of the currently available fifo */
  if(p_fifo->wr_pos > p_fifo->rd_pos)
  {
    rsize = p_fifo->max_size - p_fifo->wr_pos + p_fifo->rd_pos;
  }
  else
  {
    rsize = p_fifo->rd_pos - p_fifo->wr_pos;
  }
  
  /* Compare the currently available fifo with the reserved fifo */
  if(rsize <= BR_MAX_FIFO_SIZE)
  {
    p_fifo->is_full = 1;
  }
  
  /* set the fifo to not empty */
  p_fifo->is_empty = 0;
  
  /* enable irq */
  __enable_irq();
  
  return 1;
}

/**
  * @brief  buffer mode pop data from fifo
  * @param  p_fifo: the fifo structure pointer
  * @param  data: stored the data pointer
  * @retval pop status 1 is OK, 0 is failed
  */
uint8_t br_buffer_line_pop(br_fifo_type *p_fifo, uint8_t *data)
{
  uint8_t *pdata = (uint8_t *)p_fifo->addr;
  uint32_t rsize;
  if(p_fifo->is_empty || p_fifo == NULL)
  {
    return 0;
  }
  
  /* copy data to buffer */
  *data = pdata[p_fifo->rd_pos];
  
  /* read pos ++ */
  p_fifo->rd_pos ++;
  
  /* check the read pos is max size */
  if(p_fifo->rd_pos >= p_fifo->max_size)
  {
    p_fifo->rd_pos = 0;
  }
  
  /* disable irq */
  __disable_irq();
  
  /* check the fifo is empty? */
  if(p_fifo->rd_pos == p_fifo->wr_pos)
  {
    p_fifo->is_full = 0;
    p_fifo->is_empty = 1;
  }
  
  /* calculates the size of the currently available fifo */
  if(p_fifo->wr_pos > p_fifo->rd_pos)
  {
    rsize = p_fifo->max_size - p_fifo->wr_pos + p_fifo->rd_pos;
  }
  else
  {
    rsize = p_fifo->rd_pos - p_fifo->wr_pos;
  }
  
  /* Compare the currently available fifo with the reserved fifo */
  if(rsize > BR_MAX_FIFO_SIZE)
  {
    p_fifo->is_full = 0;
  }
  
  /* enable irq */
  __enable_irq();
  
  return 1;
}

uint32_t fifo_push_count = 0;

/**
  * @brief  buffer mode write data to fifo
  * @param  p_fifo: the fifo structure pointer
  * @param  data: write data pointer
  * @param  length: write data length 
  * @retval the length of write data
  */
uint32_t br_buffer_line_write(br_fifo_type *p_fifo, uint8_t *data, uint32_t length)
{
  uint32_t idx = 0;
  for(idx = 0; idx < length; idx ++)
  {
    br_buffer_line_push(p_fifo, data[idx]);
  }
  return length;
}

/**
  * @brief  buffer mode read the fifo data
  * @param  p_fifo: the fifo structure pointer
  * @param  data: stored the data pointer
  * @param  length: read data length 
  * @retval the length of read data
  */
uint32_t br_buffer_line_read(br_fifo_type *p_fifo, uint8_t *data, uint32_t length)
{
  uint32_t idx = 0;
  for(idx = 0; idx < length; idx ++)
  {
    if(br_buffer_line_pop(p_fifo, &data[idx]) == 0)
    {
      break;
    }
  }
  return idx;
}


