
#include "br_i2c.h"
#include "i2c_application.h"

#define I2C_TIMEOUT                      0xFFFFFFF

#define I2Cx_ADDRESS                     0x0C

#define I2Cx_PORT                        I2C2
#define I2Cx_CLK                         CRM_I2C2_PERIPH_CLOCK
#define I2Cx_DMA                         DMA1
#define I2Cx_DMA_CLK                     CRM_DMA1_PERIPH_CLOCK

#define I2Cx_SCL_GPIO_CLK                CRM_GPIOA_PERIPH_CLOCK
#define I2Cx_SCL_GPIO_PIN                GPIO_PINS_0
#define I2Cx_SCL_GPIO_PinsSource         GPIO_PINS_SOURCE0
#define I2Cx_SCL_GPIO_PORT               GPIOA
#define I2Cx_SCL_GPIO_MUX                GPIO_MUX_4

#define I2Cx_SDA_GPIO_CLK                CRM_GPIOA_PERIPH_CLOCK
#define I2Cx_SDA_GPIO_PIN                GPIO_PINS_1
#define I2Cx_SDA_GPIO_PinsSource         GPIO_PINS_SOURCE1
#define I2Cx_SDA_GPIO_PORT               GPIOA
#define I2Cx_SDA_GPIO_MUX                GPIO_MUX_4

#define I2C_RATE_MAX_SUPPORT             6

#define I2C_WRITE_HEAD                   0x55
#define I2C_READ_HEAD                    0xAA

typedef struct
{
  uint32_t bitrate;
  uint32_t clkctrl;
}i2c_rate_type;

static i2c_rate_type i2c_rate_struct[I2C_RATE_MAX_SUPPORT] =
{
  {1000000, 0x10A0101C}, /* 1M */
  {400000, 0x20F01C31},  /* 400K */
  {200000, 0x20F07DDE},  /* 200K */
  {100000, 0x90F03030},  /* 100K */
  {50000, 0x90F06666},   /* 50K */
  {10000, 0x4170FEFE},   /* 10K */
};
static i2c_rate_type *p_rate;
static uint8_t i2c_tx_flag = 0;
static uint8_t i2c_rx_buffer[BR_MAX_FIFO_SIZE];

static i2c_handle_type hi2cx;
static void br_i2c_init(void *pdata);
static void br_i2c_deinit(void *pdata);
static uint32_t br_i2c_tx(uint8_t *data, uint32_t len);
static uint8_t br_i2c_status(void);

br_handler br_i2c_handler = 
{
  br_i2c_init,
  br_i2c_deinit,
  br_i2c_tx,
  br_fifo_write,
  br_fifo_read,
  br_i2c_status,
};

/**
  * @brief  initializes peripherals used by the i2c.
  * @param  none
  * @retval none
  */
void i2c_lowlevel_init(i2c_handle_type* hi2c)
{
  gpio_init_type gpio_init_structure;
  
  if(hi2c->i2cx == I2Cx_PORT)
  {
    /* i2c periph clock enable */
    crm_periph_clock_enable(I2Cx_CLK, TRUE);
    
    crm_periph_clock_enable(I2Cx_SCL_GPIO_CLK, TRUE);
    crm_periph_clock_enable(I2Cx_SDA_GPIO_CLK, TRUE);

    /* gpio configuration */
    gpio_pin_mux_config(I2Cx_SCL_GPIO_PORT, I2Cx_SCL_GPIO_PinsSource, I2Cx_SCL_GPIO_MUX);
    gpio_pin_mux_config(I2Cx_SDA_GPIO_PORT, I2Cx_SDA_GPIO_PinsSource, I2Cx_SDA_GPIO_MUX);

    /* configure i2c pins: scl */
    gpio_init_structure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
    gpio_init_structure.gpio_mode           = GPIO_MODE_MUX;
    gpio_init_structure.gpio_out_type       = GPIO_OUTPUT_OPEN_DRAIN;
    gpio_init_structure.gpio_pull           = GPIO_PULL_NONE;

    gpio_init_structure.gpio_pins           = I2Cx_SCL_GPIO_PIN;
    gpio_init(I2Cx_SCL_GPIO_PORT, &gpio_init_structure);

    /* configure i2c pins: sda */
    gpio_init_structure.gpio_pins           = I2Cx_SDA_GPIO_PIN;
    gpio_init(I2Cx_SDA_GPIO_PORT, &gpio_init_structure);

    /* config i2c */
    i2c_init(hi2c->i2cx, 0x0F, p_rate->clkctrl);
    i2c_own_address1_set(hi2c->i2cx, I2C_ADDRESS_MODE_7BIT, I2Cx_ADDRESS);
  }
}

/**
  * @brief  i2c initialization
  * @param  pdata: config data pointer
  * @retval none
  */
static void br_i2c_init(void *pdata)
{
  uint8_t idx = 0;
  linecoding_type *linecoding = (linecoding_type *)pdata;
  
  /* set default rate is 100K */
  p_rate = &i2c_rate_struct[3];
  
  /* find the support bitrate */
  for(idx = 0; idx < I2C_RATE_MAX_SUPPORT; idx ++)
  {
    if(i2c_rate_struct[idx].bitrate == linecoding->bitrate)
    {
      p_rate = &i2c_rate_struct[idx];
      break;
    }
  }
  
  /* clear i2c tx flag */
  i2c_tx_flag = 0;
  
  /* reset i2c */
  i2c_reset(I2Cx_PORT);
  
  hi2cx.i2cx = I2Cx_PORT;
  
   /* i2c config */
  i2c_config(&hi2cx);
  
}

static void br_i2c_deinit(void *pdata)
{
  gpio_init_type gpio_init_structure;

  /* configure i2c pins: scl */
  gpio_init_structure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_structure.gpio_mode           = GPIO_MODE_INPUT;
  gpio_init_structure.gpio_out_type       = GPIO_OUTPUT_OPEN_DRAIN;
  gpio_init_structure.gpio_pull           = GPIO_PULL_NONE;

  gpio_init_structure.gpio_pins           = I2Cx_SCL_GPIO_PIN;
  gpio_init(I2Cx_SCL_GPIO_PORT, &gpio_init_structure);

  /* configure i2c pins: sda */
  gpio_init_structure.gpio_pins           = I2Cx_SDA_GPIO_PIN;
  gpio_init(I2Cx_SDA_GPIO_PORT, &gpio_init_structure);
  /* reset i2c */
  i2c_reset(I2Cx_PORT);
  
  /* i2c periph clock disable */
  crm_periph_clock_enable(I2Cx_CLK, FALSE);
  
  /* clear i2c tx flag */
  i2c_tx_flag = 0;
}

/**
  * @brief  start transfer in poll mode or interrupt mode.
  * @param  hi2c: the handle points to the operation information.
  * @param  address: slave address.
  * @param  start: config gen start condition mode.
  *         parameter as following values:
  *         - I2C_WITHOUT_START: transfer data without start condition.
  *         - I2C_GEN_START_READ: read data and generate start.
  *         - I2C_GEN_START_WRITE: send data and generate start.
  * @retval i2c status.
  */
void br_i2c_start_transfer(i2c_handle_type* hi2c, uint16_t address, i2c_start_mode_type start)
{
  if (hi2c->pcount > MAX_TRANSFER_CNT)
  {
    hi2c->psize = MAX_TRANSFER_CNT;

    i2c_transmit_set(hi2c->i2cx, address, hi2c->psize, I2C_RELOAD_MODE, start);
  }
  else
  {
    hi2c->psize = hi2c->pcount;

    i2c_transmit_set(hi2c->i2cx, address, hi2c->psize, I2C_AUTO_STOP_MODE, start);
  }
}

/**
  * @brief  return i2c send status
  * @param  none
  * @retval none
  */
static uint8_t br_i2c_status(void)
{
  return 1;
}

/**
  * @brief  the master transmits data through polling mode.
  * @param  hi2c: the handle points to the operation information.
  * @param  address: slave address.
  * @param  pdata: data buffer.
  * @param  size: data size.
  * @param  timeout: maximum waiting time.
  * @retval i2c status.
  */
i2c_status_type br_i2c_master_transmit(i2c_handle_type* hi2c, uint16_t address, uint8_t* pdata, uint16_t size, uint32_t timeout)
{
  /* initialization parameters */
  hi2c->pbuff = pdata;
  hi2c->pcount = size;

  hi2c->error_code = I2C_OK;

  /* wait for the busy flag to be reset */
  while(i2c_flag_get(hi2c->i2cx, I2C_BUSYF_FLAG) != RESET)
  {
    /* re-init */
    if(i2c_tx_flag == 0)
    {
      return I2C_ERR_STEP_1;
    }
  }
  /* start transfer */
  br_i2c_start_transfer(hi2c, address, I2C_GEN_START_WRITE);

  while (hi2c->pcount > 0)
  {
    /* wait for the tdis flag to be set */
    while((i2c_flag_get(hi2c->i2cx, I2C_TDIS_FLAG) == RESET))
    {
      if(i2c_flag_get(hi2c->i2cx, I2C_ACKFAIL_FLAG) != RESET)
      {
        /* clear ack fail flag */
        i2c_flag_clear(hi2c->i2cx, I2C_ACKFAIL_FLAG);

        hi2c->error_code = I2C_ERR_ACKFAIL;

        return I2C_ERR_STEP_2;
      }
      /* re-init */
      if(i2c_tx_flag == 0)
      {
        return I2C_ERR_STEP_2;
      }
    }

    /* send data */
    i2c_data_send(hi2c->i2cx, *hi2c->pbuff++);
    hi2c->psize--;
    hi2c->pcount--;

    if ((hi2c->psize == 0) && (hi2c->pcount != 0))
    {
      /* wait for the tcrld flag to be set  */
      while((i2c_flag_get(hi2c->i2cx, I2C_TCRLD_FLAG) == RESET))
      {
        if(i2c_flag_get(hi2c->i2cx, I2C_ACKFAIL_FLAG) != RESET)
        {
          /* clear ack fail flag */
          i2c_flag_clear(hi2c->i2cx, I2C_ACKFAIL_FLAG);

          hi2c->error_code = I2C_ERR_ACKFAIL;

          return I2C_ERR_STEP_3;
        }
        /* re-init */
        if(i2c_tx_flag == 0)
        {
          return I2C_ERR_STEP_3;
        }
    }

      /* continue transfer */
      br_i2c_start_transfer(hi2c, address, I2C_WITHOUT_START);
    }
  }

  /* wait for the stop flag to be set  */
  while(i2c_flag_get(hi2c->i2cx, I2C_STOPF_FLAG) == RESET)
  {
    /* check the ack fail flag */
    if(i2c_flag_get(hi2c->i2cx, I2C_ACKFAIL_FLAG) != RESET)
    {
      hi2c->error_code = I2C_ERR_ACKFAIL;

      return I2C_ERR_STEP_4;
    }
    
    /* re-init */
    if(i2c_tx_flag == 0)
    {
      return I2C_ERR_STEP_4;
    }
  
  }

  /* clear stop flag */
  i2c_flag_clear(hi2c->i2cx, I2C_STOPF_FLAG);

  /* reset ctrl2 register */
  i2c_reset_ctrl2_register(hi2c);

  return I2C_OK;
}

/**
  * @brief  the master receive data through polling mode.
  * @param  hi2c: the handle points to the operation information.
  * @param  address: slave address.
  * @param  pdata: data buffer.
  * @param  size: data size.
  * @param  timeout: maximum waiting time.
  * @retval i2c status.
  */
i2c_status_type br_i2c_master_receive(i2c_handle_type* hi2c, uint16_t address, uint8_t* pdata, uint16_t size, uint32_t timeout)
{
  /* initialization parameters */
  hi2c->pbuff = pdata;
  hi2c->pcount = size;

  hi2c->error_code = I2C_OK;

  /* wait for the busy flag to be reset */
  while(i2c_flag_get(hi2c->i2cx, I2C_BUSYF_FLAG) != RESET)
  {
    /* re-init */
    if(i2c_tx_flag == 0)
    {
      return I2C_ERR_STEP_1;
    }
  }
  /* start transfer */
  br_i2c_start_transfer(hi2c, address, I2C_GEN_START_READ);

  while (hi2c->pcount > 0)
  {
    /* wait for the rdbf flag to be set  */
    while((i2c_flag_get(hi2c->i2cx, I2C_RDBF_FLAG) == RESET))
    {
      if(i2c_flag_get(hi2c->i2cx, I2C_ACKFAIL_FLAG) != RESET)
      {
        /* clear ack fail flag */
        i2c_flag_clear(hi2c->i2cx, I2C_ACKFAIL_FLAG);

        hi2c->error_code = I2C_ERR_ACKFAIL;

        return I2C_ERR_STEP_2;
      }
      /* re-init */
      if(i2c_tx_flag == 0)
      {
        return I2C_ERR_STEP_2;
      }
    }

    /* read data */
    (*hi2c->pbuff++) = i2c_data_receive(hi2c->i2cx);
    hi2c->pcount--;
    hi2c->psize--;

    if ((hi2c->psize == 0) && (hi2c->pcount != 0))
    {
      /* wait for the tcrld flag to be set  */
      while((i2c_flag_get(hi2c->i2cx, I2C_TCRLD_FLAG) == RESET))
      {
        /* re-init */
        if(i2c_tx_flag == 0)
        {
          return I2C_ERR_STEP_3;
        }
      }
      /* continue transfer */
      br_i2c_start_transfer(hi2c, address, I2C_WITHOUT_START);
    }
  }

  /* wait for the stop flag to be set  */
  while(i2c_flag_get(hi2c->i2cx, I2C_STOPF_FLAG) == RESET)
  {
    /* check the ack fail flag */
    if(i2c_flag_get(hi2c->i2cx, I2C_ACKFAIL_FLAG) != RESET)
    {
      hi2c->error_code = I2C_ERR_ACKFAIL;

      return I2C_ERR_STEP_4;
    }
    /* re-init */
    if(i2c_tx_flag == 0)
    {
      return I2C_ERR_STEP_4;
    }
  
  }
  /* clear stop flag */
  i2c_flag_clear(hi2c->i2cx, I2C_STOPF_FLAG);

  /* reset ctrl2 register */
  i2c_reset_ctrl2_register(hi2c);

  return I2C_OK;
}


/**
  * @brief  i2c send and get data
  * @param  data: send data poiniter
  * @param  len: send data length
  * @retval number of data send
  */
static uint32_t br_i2c_tx(uint8_t *data, uint32_t len)
{
  /*1 byte dir + 2 byte address + 2 byte size + data */
  uint8_t dir = data[0];
  uint16_t address = data[1] << 8 | data[2];
  uint16_t size = data[3] << 8 | (data[4]);
  
  /* check the packet size */
  if(len < 5 && size > 507)
    return 0;
  
  /* set the tx flag */
  i2c_tx_flag = 1;
  
  /* wirte data */
  if(dir == I2C_WRITE_HEAD)
  {
    if((br_i2c_master_transmit(&hi2cx, address, &data[5], size, I2C_TIMEOUT)) == I2C_OK)
    {
      return size;
    }
  }
  else if(dir == I2C_READ_HEAD)
  {/* read head */
    memcpy(i2c_rx_buffer, data, 5);
    if((br_i2c_master_receive(&hi2cx, address, i2c_rx_buffer+5, size, I2C_TIMEOUT)) == I2C_OK)
    {
      /* write to fifo */
      br_i2c_handler.fifo_write(br_get_fifo(PERIPHERAL_TO_USB), i2c_rx_buffer, size + 5);
      return size;
    }
  }
  return len;
}

