#include "br_rs485.h"
#include "usb_multi_bridge.h"

#define USART_USE_INTERRUPT

static void br_rs485_init(void *pdata);
static void br_rs485_deinit(void *pdata);
static uint32_t br_rs485_tx(uint8_t *data, uint32_t len);
static uint8_t br_rs485_status(void);

#ifdef USART_USE_INTERRUPT
static uint8_t *p_tx;
static uint32_t usart_tx_counter = 0;
#endif
static uint32_t usart_tx_len = 0;


#define USART_RX_BUFFER_SIZE        8
#define USART_HALF_RX_BUFFER_SIZE   (USART_RX_BUFFER_SIZE / 2)

#ifdef USART_USE_DMA
static uint8_t usart_dma_buffer[USART_RX_BUFFER_SIZE];
static uint8_t usart_rx_buffer[USART_RX_BUFFER_SIZE];
#endif
br_handler br_rs485_handler = 
{
  br_rs485_init,
  br_rs485_deinit,
  br_rs485_tx,
  br_buffer_line_write,
  br_buffer_line_read,
  br_rs485_status,
};

/**
  * @brief  usart initialization
  * @param  pdata: config data pointer
  * @retval none
  */
static void br_rs485_init(void *pdata)
{
  gpio_init_type gpio_init_struct;
  usart_stop_bit_num_type usart_stop_bit;
  usart_data_bit_num_type usart_data_bit;
  usart_parity_selection_type usart_parity_select;
  
  linecoding_type *linecoding = (linecoding_type *)pdata;
  crm_periph_reset(CRM_USART5_PERIPH_RESET, TRUE);
  crm_periph_reset(CRM_USART5_PERIPH_RESET, FALSE);
  
  /* enable the USART5 and gpio clock */
  crm_periph_clock_enable(CRM_USART5_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_GPIOB_PERIPH_CLOCK, TRUE);
  
  /* configure the usart5 tx, rx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type  = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_MUX;
  gpio_init_struct.gpio_pins = GPIO_PINS_4 | GPIO_PINS_5 | GPIO_PINS_6;
  gpio_init_struct.gpio_pull = GPIO_PULL_UP;
  gpio_init(GPIOB, &gpio_init_struct);
  gpio_pin_mux_config(GPIOB, GPIO_PINS_SOURCE4, GPIO_MUX_10);
  gpio_pin_mux_config(GPIOB, GPIO_PINS_SOURCE5, GPIO_MUX_10);
  gpio_pin_mux_config(GPIOB, GPIO_PINS_SOURCE6, GPIO_MUX_8);
  
  /* stop bit */
  switch(linecoding->format)
  {
    case 0x0:
      usart_stop_bit = USART_STOP_1_BIT;
      break;
    /* to be used when transmitting and receiving data in smartcard mode */
    case 0x1:
      usart_stop_bit = USART_STOP_1_5_BIT;
      break;
    case 0x2:
      usart_stop_bit = USART_STOP_2_BIT;
      break;
    default :
      break;
  }

    /* parity */
  switch(linecoding->parity)
  {
    case 0x0:
      usart_parity_select = USART_PARITY_NONE;
      break;
    case 0x1:
      usart_parity_select = USART_PARITY_ODD;
      break;
    case 0x2:
      usart_parity_select = USART_PARITY_EVEN;
      break;
    /* hardware usart not support parity for mark and space  */
    case 0x3:
    case 0x4:
      break;
    default :
      break;
  }

  if(USART_PARITY_NONE == usart_parity_select)
  {
    /* data bits */
    switch(linecoding->data)
    {
      /* hardware usart not support data bits for 5/6 */
      case 0x5:
      case 0x6:
        break;
      case 0x7:
        usart_data_bit = USART_DATA_7BITS;
        break;
      case 0x8:
        usart_data_bit = USART_DATA_8BITS;
        break;
      /* hardware usart not support data bits for 16 */
      case 0x10:
        break;
      default :
        break;
    }
  }
  else
  {
    /* data bits */
    switch(linecoding->data)
    {
      /* hardware usart not support data bits for 5/6 */
      case 0x5:
      case 0x6:
        break;
      case 0x7:
        usart_data_bit = USART_DATA_8BITS;
        break;
      case 0x8:
        usart_data_bit = USART_DATA_9BITS;
        break;
      /* hardware usart not support data bits for 16 */
      case 0x10:
        break;
      default :
        break;
    }
  }
  
  usart_tx_len = 0;
  /* configure USART5 param */
  usart_init(USART5, linecoding->bitrate, usart_data_bit, usart_stop_bit);
  usart_rs485_delay_time_config(USART5, 2, 2);
  usart_de_polarity_set(USART5, USART_DE_POLARITY_HIGH);
  usart_rs485_mode_enable(USART5, TRUE);
  
  usart_transmitter_enable(USART5, TRUE);
  usart_receiver_enable(USART5, TRUE);
  
  nvic_irq_enable(USART5_IRQn, 1, 0);
#ifdef USART_USE_INTERRUPT
  usart_flag_clear(USART5, USART_RDBF_FLAG);
  usart_interrupt_enable(USART5, USART_RDBF_INT, TRUE);
#endif

  usart_enable(USART5, TRUE);
}


static void br_rs485_deinit(void *pdata)
{
  gpio_init_type gpio_init_struct;
  crm_periph_reset(CRM_USART5_PERIPH_RESET, TRUE);
  crm_periph_reset(CRM_USART5_PERIPH_RESET, FALSE);
  
  /* enable the USART5 and gpio clock */
  crm_periph_clock_enable(CRM_USART5_PERIPH_CLOCK, FALSE);
  
  /* configure the usart5 tx, rx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type  = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_INPUT;
  gpio_init_struct.gpio_pins = GPIO_PINS_4 | GPIO_PINS_5 | GPIO_PINS_6;
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE;
  gpio_init(GPIOB, &gpio_init_struct);
}

/**
  * @brief  return usart dma send status
  * @param  none
  * @retval none
  */
static uint8_t br_rs485_status(void)
{
  if(usart_tx_len != 0)
    return 0;
  else
    return 1;
}

/**
  * @brief  USART5 irq
  * @param  none
  * @retval none
  */
void USART5_IRQHandler(void)
{
#ifdef USART_USE_INTERRUPT
  if(usart_interrupt_flag_get(USART5, USART_RDBF_FLAG) != RESET)
  {
    /* read one byte from the receive data register */
    br_buffer_line_push(br_get_fifo(PERIPHERAL_TO_USB), usart_data_receive(USART5));
  }

  if(usart_interrupt_flag_get(USART5, USART_TDBE_FLAG) != RESET)
  {
    usart_data_transmit(USART5, p_tx[usart_tx_counter++]);
    if(usart_tx_counter >= usart_tx_len)
    {
      usart_interrupt_enable(USART5, USART_TDBE_INT, FALSE);
      usart_tx_len = 0;
    }
  }
#endif
}

/**
  * @brief  usart send interface
  * @param  data: send data poiniter
  * @param  len: send data length
  * @retval number of data need send
  */
static uint32_t br_rs485_tx(uint8_t *data, uint32_t len)
{
  if(usart_tx_len != 0)
    return 0;
  
  usart_tx_len = len;
#ifdef USART_USE_INTERRUPT
  p_tx = data;
  usart_tx_counter = 0;
#endif
#ifdef USART_USE_INTERRUPT
  usart_interrupt_enable(USART5, USART_TDBE_INT, TRUE);
#endif
  
  return len;
}

