
#include "br_spi.h"

#define SPI_RATE_MAX_SUPPORT  6

typedef struct
{
  uint32_t bitrate;
  spi_mclk_freq_div_type div;
}spi_rate_type;

/* apb 216m */
static spi_rate_type spi_rate_struct[SPI_RATE_MAX_SUPPORT] =
{
  {27000000, SPI_MCLK_DIV_8},   /* 27M */
  {13500000, SPI_MCLK_DIV_16},  /* 13.5M */
  {6750000, SPI_MCLK_DIV_32},   /* 6.75M */
  {3375000, SPI_MCLK_DIV_64},   /* 3.375M */
  {1687500, SPI_MCLK_DIV_128},  /* 1.6875M */
  {843750, SPI_MCLK_DIV_256},  /* 0.84375M */
};

#define SPI_CS_HIGH()                  gpio_bits_set(GPIOA, GPIO_PINS_4)
#define SPI_CS_LOW()                   gpio_bits_reset(GPIOA, GPIO_PINS_4)

static void br_spi_init(void *pdata);
static void br_spi_deinit(void *pdata);
static uint32_t br_spi_tx(uint8_t *data, uint32_t len);
static uint8_t spi_byte_write(uint8_t data);
static uint8_t br_spi_status(void);

static uint8_t spi_rx_buf[BR_MAX_FIFO_SIZE];
static uint8_t spi_tx_flag = 0;

br_handler br_spi_handler = 
{
  br_spi_init,
  br_spi_deinit,
  br_spi_tx,
  br_fifo_write,
  br_fifo_read,
  br_spi_status,
};

/**
  * @brief  spi initialization
  * @param  pdata: config data pointer
  * @retval none
  */
static void br_spi_init(void *pdata)
{
  gpio_init_type gpio_initstructure;
  spi_init_type spi_init_struct;
  spi_rate_type *p_rate = &spi_rate_struct[4];
  uint8_t idx = 0;
  linecoding_type *linecoding = (linecoding_type *)pdata;
  
  /* clear spi tx flag */
  spi_tx_flag = 0;
  
  /* reset spi */
  spi_i2s_reset(SPI1);
  
  /* find support rate */
  for(idx = 0; idx < SPI_RATE_MAX_SUPPORT; idx ++)
  {
    if(spi_rate_struct[idx].bitrate == linecoding->bitrate)
    {
      p_rate = &spi_rate_struct[idx];
      break;
    }
  }

  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);

  /* software cs, pa4 as a general io to control flash cs */
  SPI_CS_HIGH();
  gpio_initstructure.gpio_out_type       = GPIO_OUTPUT_PUSH_PULL;
  gpio_initstructure.gpio_pull           = GPIO_PULL_UP;
  gpio_initstructure.gpio_mode           = GPIO_MODE_OUTPUT;
  gpio_initstructure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_initstructure.gpio_pins           = GPIO_PINS_4;
  gpio_init(GPIOA, &gpio_initstructure);

  /* sck */
  gpio_initstructure.gpio_pull           = GPIO_PULL_UP;
  gpio_initstructure.gpio_mode           = GPIO_MODE_MUX;
  gpio_initstructure.gpio_pins           = GPIO_PINS_5;
  gpio_init(GPIOA, &gpio_initstructure);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE5, GPIO_MUX_5);

  /* miso */
  gpio_initstructure.gpio_pull           = GPIO_PULL_UP;
  gpio_initstructure.gpio_pins           = GPIO_PINS_6;
  gpio_init(GPIOA, &gpio_initstructure);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE6, GPIO_MUX_5);

  /* mosi */
  gpio_initstructure.gpio_pull           = GPIO_PULL_UP;
  gpio_initstructure.gpio_pins           = GPIO_PINS_7;
  gpio_init(GPIOA, &gpio_initstructure);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE7, GPIO_MUX_5);
  
  gpio_pins_huge_driven_config(GPIOA, GPIO_PINS_5, TRUE);

  crm_periph_clock_enable(CRM_SPI1_PERIPH_CLOCK, TRUE);
  spi_default_para_init(&spi_init_struct);
  spi_init_struct.transmission_mode = SPI_TRANSMIT_FULL_DUPLEX;
  spi_init_struct.master_slave_mode = SPI_MODE_MASTER;
  spi_init_struct.mclk_freq_division = p_rate->div;
  spi_init_struct.first_bit_transmission = SPI_FIRST_BIT_MSB;
  spi_init_struct.frame_bit_num = SPI_FRAME_8BIT;
  spi_init_struct.clock_polarity = SPI_CLOCK_POLARITY_HIGH;
  spi_init_struct.clock_phase = SPI_CLOCK_PHASE_2EDGE;
  spi_init_struct.cs_mode_selection = SPI_CS_SOFTWARE_MODE;
  spi_init(SPI1, &spi_init_struct);

  spi_enable(SPI1, TRUE);

}

static void br_spi_deinit(void *pdata)
{
  gpio_init_type gpio_initstructure;

  gpio_initstructure.gpio_out_type       = GPIO_OUTPUT_PUSH_PULL;
  gpio_initstructure.gpio_pull           = GPIO_PULL_NONE;
  gpio_initstructure.gpio_mode           = GPIO_MODE_INPUT;
  gpio_initstructure.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_initstructure.gpio_pins           = GPIO_PINS_4;
  gpio_init(GPIOA, &gpio_initstructure);

  /* sck */
  gpio_initstructure.gpio_pull           = GPIO_PULL_NONE;
  gpio_initstructure.gpio_mode           = GPIO_MODE_INPUT;
  gpio_initstructure.gpio_pins           = GPIO_PINS_5;
  gpio_init(GPIOA, &gpio_initstructure);
  
  /* miso */
  gpio_initstructure.gpio_pull           = GPIO_PULL_NONE;
  gpio_initstructure.gpio_pins           = GPIO_PINS_6;
  gpio_init(GPIOA, &gpio_initstructure);

  /* mosi */
  gpio_initstructure.gpio_pull           = GPIO_PULL_NONE;
  gpio_initstructure.gpio_pins           = GPIO_PINS_7;
  gpio_init(GPIOA, &gpio_initstructure);
  
  /* reset spi */
  spi_i2s_reset(SPI1);
  
  crm_periph_clock_enable(CRM_SPI1_PERIPH_CLOCK, FALSE);
  
  /* clear spi tx flag */
  spi_tx_flag = 0;
  
}

/**
  * @brief  return spi send status
  * @param  none
  * @retval none
  */
static uint8_t br_spi_status(void)
{
  return 1;
}

/**
  * @brief  spi send interface
  * @param  data: send data poiniter
  * @param  len: send data length
  * @retval number of data send
  */
static uint32_t br_spi_tx(uint8_t *data, uint32_t len)
{
  uint32_t idx = 0;
  
  /* set tx flag */
  spi_tx_flag = 1;
  
  /* start tx data */
  SPI_CS_LOW();
  for(idx = 0; idx < len; idx ++)
  {
    spi_rx_buf[idx] = spi_byte_write(data[idx]);
  }
  SPI_CS_HIGH();
  
  /* write data to fifo */
  if(spi_tx_flag)
  {
    br_spi_handler.fifo_write(br_get_fifo(PERIPHERAL_TO_USB), spi_rx_buf, idx);
  }
  return idx;
}

/**
  * @brief  spi master send one byte
  * @param  data: send data
  * @retval receive data
  */
static uint8_t spi_byte_write(uint8_t data)
{
  uint8_t tmp;
  uint32_t timeout = 0xFFFFF;
  
  /* check tx flag */
  if(spi_tx_flag == 0)
  {
    return 0xFF;
  }
  
  /* transmit data */
  spi_i2s_data_transmit(SPI1, data);
  
  /* wait transmit complete */
  while(spi_i2s_flag_get(SPI1, SPI_I2S_RDBF_FLAG) == RESET && timeout -- && spi_tx_flag);
  
  /* receivce data */
  tmp = spi_i2s_data_receive(SPI1);
  
  /* wait busy flag clear */
  while(spi_i2s_flag_get(SPI1, SPI_I2S_BF_FLAG) != RESET && spi_tx_flag);
  
  return tmp;
}
