#include "br_usart.h"
#include "usb_multi_bridge.h"

//#define USART_USE_INTERRUPT
#define USART_USE_DMA

static void br_usart_init(void *pdata);
static void br_usart_deinit(void *pdata);
static uint32_t br_usart_tx(uint8_t *data, uint32_t len);
static uint8_t br_usart_status(void);

#ifdef USART_USE_DMA
static void usart_dma_configuration(void);
#endif

#ifdef USART_USE_INTERRUPT
static uint8_t *p_tx;
static uint32_t usart_tx_counter = 0;
#endif
static uint32_t usart_tx_len = 0;


#define USART_RX_BUFFER_SIZE        8
#define USART_HALF_RX_BUFFER_SIZE   (USART_RX_BUFFER_SIZE / 2)

#ifdef USART_USE_DMA
static uint8_t usart_dma_buffer[USART_RX_BUFFER_SIZE];
static uint8_t usart_rx_buffer[USART_RX_BUFFER_SIZE];
#endif
br_handler br_usart_handler = 
{
  br_usart_init,
  br_usart_deinit,
  br_usart_tx,
  br_buffer_line_write,
  br_buffer_line_read,
  br_usart_status,
};

/**
  * @brief  usart initialization
  * @param  pdata: config data pointer
  * @retval none
  */
static void br_usart_init(void *pdata)
{
  gpio_init_type gpio_init_struct;
  usart_stop_bit_num_type usart_stop_bit;
  usart_data_bit_num_type usart_data_bit;
  usart_parity_selection_type usart_parity_select;
  
  linecoding_type *linecoding = (linecoding_type *)pdata;
  crm_periph_reset(CRM_USART2_PERIPH_RESET, TRUE);
  crm_periph_reset(CRM_USART2_PERIPH_RESET, FALSE);
  
  /* enable the usart2 and gpio clock */
  crm_periph_clock_enable(CRM_USART2_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);
  
  /* configure the usart2 tx, rx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type  = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_MUX;
  gpio_init_struct.gpio_pins = GPIO_PINS_2 | GPIO_PINS_3;
  gpio_init_struct.gpio_pull = GPIO_PULL_UP;
  gpio_init(GPIOA, &gpio_init_struct);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE2, GPIO_MUX_7);
  gpio_pin_mux_config(GPIOA, GPIO_PINS_SOURCE3, GPIO_MUX_7);
  
  /* stop bit */
  switch(linecoding->format)
  {
    case 0x0:
      usart_stop_bit = USART_STOP_1_BIT;
      break;
    /* to be used when transmitting and receiving data in smartcard mode */
    case 0x1:
      usart_stop_bit = USART_STOP_1_5_BIT;
      break;
    case 0x2:
      usart_stop_bit = USART_STOP_2_BIT;
      break;
    default :
      break;
  }

    /* parity */
  switch(linecoding->parity)
  {
    case 0x0:
      usart_parity_select = USART_PARITY_NONE;
      break;
    case 0x1:
      usart_parity_select = USART_PARITY_ODD;
      break;
    case 0x2:
      usart_parity_select = USART_PARITY_EVEN;
      break;
    /* hardware usart not support parity for mark and space  */
    case 0x3:
    case 0x4:
      break;
    default :
      break;
  }

  if(USART_PARITY_NONE == usart_parity_select)
  {
    /* data bits */
    switch(linecoding->data)
    {
      /* hardware usart not support data bits for 5/6 */
      case 0x5:
      case 0x6:
        break;
      case 0x7:
        usart_data_bit = USART_DATA_7BITS;
        break;
      case 0x8:
        usart_data_bit = USART_DATA_8BITS;
        break;
      /* hardware usart not support data bits for 16 */
      case 0x10:
        break;
      default :
        break;
    }
  }
  else
  {
    /* data bits */
    switch(linecoding->data)
    {
      /* hardware usart not support data bits for 5/6 */
      case 0x5:
      case 0x6:
        break;
      case 0x7:
        usart_data_bit = USART_DATA_8BITS;
        break;
      case 0x8:
        usart_data_bit = USART_DATA_9BITS;
        break;
      /* hardware usart not support data bits for 16 */
      case 0x10:
        break;
      default :
        break;
    }
  }
  
  usart_tx_len = 0;
  /* configure usart2 param */
  usart_init(USART2, linecoding->bitrate, usart_data_bit, usart_stop_bit);
  usart_parity_selection_config(USART2, usart_parity_select);
  usart_transmitter_enable(USART2, TRUE);
  usart_receiver_enable(USART2, TRUE);
  
  nvic_irq_enable(USART2_IRQn, 1, 0);
#ifdef USART_USE_INTERRUPT  
  usart_interrupt_enable(USART2, USART_RDBF_INT, TRUE);
//  usart_interrupt_enable(USART2, USART_TDBE_INT, TRUE);
#endif
#ifdef USART_USE_DMA
  usart_interrupt_enable(USART2, USART_IDLE_INT, TRUE);
  usart_dma_receiver_enable(USART2, TRUE);
  usart_dma_transmitter_enable(USART2, TRUE);
  usart_dma_configuration();
#endif

  usart_enable(USART2, TRUE);
}

/**
  * @brief  usart initialization
  * @param  pdata: config data pointer
  * @retval none
  */
static void br_usart_deinit(void *pdata)
{
  gpio_init_type gpio_init_struct;
  crm_periph_reset(CRM_USART2_PERIPH_RESET, TRUE);
  crm_periph_reset(CRM_USART2_PERIPH_RESET, FALSE);
  
  /* enable the usart2 and gpio clock */
  crm_periph_clock_enable(CRM_USART2_PERIPH_CLOCK, FALSE);
  
  /* set tx, rx floating */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type  = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_INPUT;
  gpio_init_struct.gpio_pins = GPIO_PINS_2 | GPIO_PINS_3;
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE;
  gpio_init(GPIOA, &gpio_init_struct);
  
  usart_tx_len = 0;
}

#ifdef USART_USE_DMA
/**
  * @brief  usart dma receive irq
  * @param  none
  * @retval none
  */
void DMA1_Channel2_IRQHandler(void)
{
  if(dma_interrupt_flag_get(DMA1_FDT2_FLAG))
  {
    dma_flag_clear(DMA1_FDT2_FLAG);
    
    DMA1_CHANNEL2->ctrl_bit.chen = FALSE;
    memcpy(usart_rx_buffer, usart_dma_buffer + USART_HALF_RX_BUFFER_SIZE,  USART_RX_BUFFER_SIZE / 2);
    
    /* re-enable dma transfer */
    DMA1_CHANNEL2->dtcnt = USART_RX_BUFFER_SIZE;
    DMA1_CHANNEL2->ctrl_bit.chen = TRUE;
    
    /* write data to fifo */
    br_usart_handler.fifo_write(br_get_fifo(PERIPHERAL_TO_USB), usart_rx_buffer, USART_RX_BUFFER_SIZE / 2);
  }
  if(dma_interrupt_flag_get(DMA1_HDT2_FLAG))
  {
    dma_flag_clear(DMA1_HDT2_FLAG);
    memcpy(usart_rx_buffer, usart_dma_buffer,  USART_RX_BUFFER_SIZE / 2);
    
    /* write data to fifo */
    br_usart_handler.fifo_write(br_get_fifo(PERIPHERAL_TO_USB), usart_rx_buffer, USART_RX_BUFFER_SIZE / 2);
  }
}

/**
  * @brief  usart dma send irq
  * @param  none
  * @retval none
  */
void DMA1_Channel1_IRQHandler(void)
{
  if(dma_interrupt_flag_get(DMA1_FDT1_FLAG))
  {
    dma_flag_clear(DMA1_FDT1_FLAG);
    usart_tx_len = 0;
    dma_channel_enable(DMA1_CHANNEL1, FALSE);
  }
}

/**
  * @brief  usart dma config
  * @param  none
  * @retval none
  */
static void usart_dma_configuration(void)
{
  dma_init_type dma_init_struct;

  /* enable dma1 clock */
  crm_periph_clock_enable(CRM_DMA1_PERIPH_CLOCK, TRUE);

  dmamux_enable(DMA1, TRUE);
  
  /* dma1 channel2 for usart2 rx configuration */
  dma_reset(DMA1_CHANNEL2);
  dma_default_para_init(&dma_init_struct);
  dma_init_struct.buffer_size = USART_RX_BUFFER_SIZE;
  dma_init_struct.direction = DMA_DIR_PERIPHERAL_TO_MEMORY;
  dma_init_struct.memory_base_addr = (uint32_t)usart_dma_buffer;
  dma_init_struct.memory_data_width = DMA_MEMORY_DATA_WIDTH_BYTE;
  dma_init_struct.memory_inc_enable = TRUE;
  dma_init_struct.peripheral_base_addr = (uint32_t)&USART2->dt;
  dma_init_struct.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_BYTE;
  dma_init_struct.peripheral_inc_enable = FALSE;
  dma_init_struct.priority = DMA_PRIORITY_MEDIUM;
  dma_init_struct.loop_mode_enable = FALSE;
  dma_init(DMA1_CHANNEL2, &dma_init_struct);

  /* enable transfer full data interrupt */
  dma_interrupt_enable(DMA1_CHANNEL2, DMA_FDT_INT, TRUE);
  dma_interrupt_enable(DMA1_CHANNEL2, DMA_HDT_INT, TRUE);

  /* dma1 channel2 interrupt nvic init */
  nvic_irq_enable(DMA1_Channel2_IRQn, 1, 0);

  /* config flexible dma for usart2 rx */
  dmamux_init(DMA1MUX_CHANNEL2, DMAMUX_DMAREQ_ID_USART2_RX);
  
  dma_channel_enable(DMA1_CHANNEL2, TRUE); /* usart2 rx begin dma transmitting */
  

  /* dma1 channel1 for usart2 tx configuration */
  dma_reset(DMA1_CHANNEL1);
  dma_default_para_init(&dma_init_struct);
  dma_init_struct.buffer_size = 0;
  dma_init_struct.direction = DMA_DIR_MEMORY_TO_PERIPHERAL;
  dma_init_struct.memory_base_addr = (uint32_t)0;
  dma_init_struct.memory_data_width = DMA_MEMORY_DATA_WIDTH_BYTE;
  dma_init_struct.memory_inc_enable = TRUE;
  dma_init_struct.peripheral_base_addr = (uint32_t)&USART2->dt;
  dma_init_struct.peripheral_data_width = DMA_PERIPHERAL_DATA_WIDTH_BYTE;
  dma_init_struct.peripheral_inc_enable = FALSE;
  dma_init_struct.priority = DMA_PRIORITY_MEDIUM;
  dma_init_struct.loop_mode_enable = FALSE;
  dma_init(DMA1_CHANNEL1, &dma_init_struct);

  /* enable transfer full data interrupt */
  dma_interrupt_enable(DMA1_CHANNEL1, DMA_FDT_INT, TRUE);

  /* dma1 channel1 interrupt nvic init */
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_4);
  nvic_irq_enable(DMA1_Channel1_IRQn, 2, 0);

  /* config flexible dma for usart2 tx */
  dmamux_init(DMA1MUX_CHANNEL1, DMAMUX_DMAREQ_ID_USART2_TX);
  
}
#endif

/**
  * @brief  return usart dma send status
  * @param  none
  * @retval none
  */
static uint8_t br_usart_status(void)
{
  if(usart_tx_len != 0)
    return 0;
  else
    return 1;
}

/**
  * @brief  usart2 irq
  * @param  none
  * @retval none
  */
void USART2_IRQHandler(void)
{
#ifdef USART_USE_INTERRUPT
  if(usart_interrupt_flag_get(USART2, USART_RDBF_FLAG) != RESET)
  {
    /* read one byte from the receive data register */
    br_buffer_line_push(br_get_fifo(PERIPHERAL_TO_USB), usart_data_receive(USART2));

  }

  if(usart_interrupt_flag_get(USART2, USART_TDBE_FLAG) != RESET)
  {
    usart_data_transmit(USART2, p_tx[usart_tx_counter++]);
    if(usart_tx_counter >= usart_tx_len)
    {
      usart_interrupt_enable(USART2, USART_TDBE_INT, FALSE);
      usart_tx_len = 0;
    }
  }
#endif

#ifdef USART_USE_DMA
  /* idle interrupt */
  if(usart_interrupt_flag_get(USART2, USART_IDLEF_FLAG) != RESET)
  {
    uint16_t count = USART_RX_BUFFER_SIZE - DMA1_CHANNEL2->dtcnt;
    /* disable dma */
    DMA1_CHANNEL2->ctrl_bit.chen = FALSE;
    
    /* copy data */
    if(count > USART_HALF_RX_BUFFER_SIZE)
    {
      count = count - USART_HALF_RX_BUFFER_SIZE;
      memcpy(usart_rx_buffer, usart_dma_buffer + USART_HALF_RX_BUFFER_SIZE, count);
    }
    else if(count == USART_HALF_RX_BUFFER_SIZE)
    {
      if(dma_interrupt_flag_get(DMA1_HDT2_FLAG))
      {
        count = count - USART_HALF_RX_BUFFER_SIZE;
        memcpy(usart_rx_buffer, usart_dma_buffer + USART_HALF_RX_BUFFER_SIZE, count);
      }
      else
      {
        count = 0;
      }
    }
    else
    {
      memcpy(usart_rx_buffer, usart_dma_buffer, count);
    }
    
    /* read data register */
    usart_data_receive(USART2);
    
    /* clear dma flag */
    dma_flag_clear(DMA1_FDT2_FLAG);
    dma_flag_clear(DMA1_HDT2_FLAG);
    
    /* set dma counter */
    DMA1_CHANNEL2->dtcnt = USART_RX_BUFFER_SIZE;
    DMA1_CHANNEL2->ctrl_bit.chen = TRUE;
    
    /* write data to fifo */
    br_usart_handler.fifo_write(br_get_fifo(PERIPHERAL_TO_USB), usart_rx_buffer, count);
  }
#endif
}

/**
  * @brief  usart send interface
  * @param  data: send data poiniter
  * @param  len: send data length
  * @retval number of data need send
  */
static uint32_t br_usart_tx(uint8_t *data, uint32_t len)
{
  if(usart_tx_len != 0)
    return 0;
  
  usart_tx_len = len;
#ifdef USART_USE_INTERRUPT
  p_tx = data;
  usart_tx_counter = 0;
#endif
#ifdef USART_USE_DMA
  DMA1_CHANNEL1->dtcnt = len;
  DMA1_CHANNEL1->maddr = (uint32_t)data;
  DMA1_CHANNEL1->ctrl_bit.chen = TRUE;
#endif
#ifdef USART_USE_INTERRUPT
  usart_interrupt_enable(USART2, USART_TDBE_INT, TRUE);
#endif
  
  return len;
}

