
#include "usb_multi_bridge.h"
#include "at32f402_405_board.h"

/* support peripheral */
extern br_handler br_usart_handler;
extern br_handler br_can_handler;
extern br_handler br_spi_handler;
extern br_handler br_i2c_handler;
extern br_handler br_rs485_handler;

/* usb multi struct */
static usb_multi_bridge_type usb_br = {USB_TO_IDLE, NULL};
static usb_multi_bridge_type *pbr = &usb_br;

/* usb multi bridge fifo */
static br_fifo_type usb_rx_fifo;
static br_fifo_type usb_tx_fifo;

/* usb multi bridge buffer */
static uint32_t tx_buffer[BR_MAX_FIFO_SIZE];
static uint32_t rx_buffer[BR_MAX_FIFO_SIZE];

/*
#define PERIPH_CONFIG_LED()        at32_led_on(LED3)
#define PERIPH_UNCONFIG_LED()      at32_led_off(LED4)
*/
#define PERIPH_CONFIG_LED()
#define PERIPH_UNCONFIG_LED()
/**
  * @brief  usb recv callback function
  * @param  data: usb recv data pointer
  * @param  length: usb recv data length
  * @retval the fifo full flag
  */
uint8_t cdc_recv_callback(uint8_t *data, uint32_t length)
{
  br_fifo_type *p_fifo = br_get_fifo(USB_TO_PERIPHERAL);
  pbr->handler->fifo_write(p_fifo, data, length);
  return p_fifo->is_full;
}

/**
  * @brief  usb multi bridge process
  * @param  pudev: usb device core type
  * @retval none
  */
void usb_multi_bridge_process(usbd_core_type *pudev)
{
  uint32_t length;
  br_fifo_type *p_fifo;
  
  cdc_struct_type *pcdc = (cdc_struct_type *)pudev->class_handler->pdata;
  
  if(pbr->handler == 0)
    return;
  
  /* usb receive flow control */
  p_fifo = br_get_fifo(USB_TO_PERIPHERAL);
  if((pcdc->flow_control == 1) && (p_fifo->is_full == 0))
  {
    pcdc->flow_control = 0;
    usbd_ept_recv(pudev, USBD_CDC_BULK_OUT_EPT, pcdc->g_rx_buff, pcdc->maxpacket);
  }
  
  /* usb to peripheral */
  if(pbr->handler->br_tx_status())
  {
    length = pbr->handler->fifo_read(p_fifo, (uint8_t *)rx_buffer, BR_MAX_FIFO_SIZE);
    if(length > 0)
    {
      pbr->handler->br_tx((uint8_t *)rx_buffer, length);
    }
  }
  
  /* peripheral to usb */
  if(pcdc->g_tx_completed == 1)
  {
    length = pbr->handler->fifo_read(br_get_fifo(PERIPHERAL_TO_USB), (uint8_t *)tx_buffer, BR_MAX_FIFO_SIZE);
    if(length > 0)
    {
      usb_vcp_send_data(pudev, (uint8_t *)tx_buffer, length);
    }
  }
  
  return;
}

/**
  * @brief  usb send data complete callback
  * @param  pudev: usb device core type
  * @retval none
  */
uint8_t usb_in_complete_callback(usbd_core_type *pudev)
{
  uint32_t length;
  length = pbr->handler->fifo_read(br_get_fifo(PERIPHERAL_TO_USB), (uint8_t *)tx_buffer, BR_MAX_FIFO_SIZE);
  if(length > 0)
  {
    usb_vcp_send_data(pudev, (uint8_t *)tx_buffer, length);
  }
  return length;
}

/**
  * @brief  get the fifo based on the direction 
  * @param  dir: the direction
  * @retval the fifo pointer
  */
br_fifo_type *br_get_fifo(trans_dir_enum dir)
{
  if(dir == USB_TO_PERIPHERAL)
    return &usb_rx_fifo;
  else
    return &usb_tx_fifo;
}

/**
  * @brief  get the currently usb to peripheral type
  * @param  none
  * @retval none
  */
_usb_multi_bridge_enum get_usb_to_type(void)
{
  return  pbr->br_type;
}

/**
  * @brief  set the currently usb to peripheral type
  * @param  br_type: peripheral type
  * @retval none
  */
void usb_multi_bridge_set_type(_usb_multi_bridge_enum br_type)
{
  if(pbr->br_type != USB_TO_IDLE && pbr->handler != NULL)
  {
    pbr->handler->deinit(NULL);
  }
  
  pbr->br_type = br_type;
  switch(pbr->br_type)
  {
    case USB_TO_USART:
      pbr->handler = &br_usart_handler;
      break;
    case USB_TO_CAN:
      pbr->handler = &br_can_handler;
      break;
    case USB_TO_SPI:
      pbr->handler = &br_spi_handler;
      break;
    case USB_TO_I2C:
      pbr->handler = &br_i2c_handler;
      break;
    case USB_TO_RS485:
      pbr->handler = &br_rs485_handler;
      break;
    default:
      pbr->br_type = USB_TO_IDLE;
      pbr->handler = NULL;
      break;
  };
  
  PERIPH_UNCONFIG_LED();
  
  /* initialization fifo */
  br_fifo_init(br_get_fifo(USB_TO_PERIPHERAL));
  br_fifo_init(br_get_fifo(PERIPHERAL_TO_USB));
}

/**
  * @brief  initialization the bridge peripheral and fifo
  * @param  pdata: the initializes data
  * @retval none
  */
void usb_multi_bridge_init(void *pdata)
{
  /* initialization fifo */
  br_fifo_init(br_get_fifo(USB_TO_PERIPHERAL));
  br_fifo_init(br_get_fifo(PERIPHERAL_TO_USB));
  
  PERIPH_CONFIG_LED();
  
  /* peripheral init */
  if(pbr->handler != NULL)
  {
    pbr->handler->init(pdata);
  }
  
}

